/**
 * DirectoryWalker runs through the source directory and apply the
 * styling as appropriate to create the target directory.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

package org.ananas.xm;

import java.io.*;
import java.util.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import org.ananas.util.NotImplementedException;

public class DirectoryWalker
{
   /**
    * the "compiled" style sheet
    */
   protected Templates templates;

   /**
    * the time that the style sheet was last modified
    */
   protected long lastModified;

   /**
    * extension derived from the xsl:output's method attribute
    */
   protected String extension;

   /**
    * output messages on this messager
    */
   protected Messager messager = null;

   /**
    * creates a new DirectoryWalker
    * @param stylesheetFile the style sheet to apply
    * @param messager messager to report progress and error messages to
    */
   public DirectoryWalker(File stylesheetFile,Messager messager)
      throws TransformerConfigurationException
   {
      lastModified = stylesheetFile.lastModified();
      templates = TransformerFactory.newInstance().newTemplates(new StreamSource(stylesheetFile));
      extension = '.' + templates.getOutputProperties().getProperty(OutputKeys.METHOD);
      this.messager = messager;
   }

   /*
   un object Styler (interface) qui dfinit un target aussi...
   le target tant une cible (*.xml, *.pdf, etc.), on peut obtenir les cibles et voir ce qui se passe
   */

   public void walk(String source,String target)
      throws IOException, XMException
   {
      walk(new File(source),new File(target));
   }

   protected void walk(File source,File target)
      throws IOException, XMException
   {
      if(source.isDirectory())
      {
         // File rulesFile = new File(source,Constants.rulesFilename);
         File[] files = source.listFiles(),
                dirs = new File[files.length],
                docs = new File[files.length];
         int idirs = 0,
             idocs = 0;
         for(int i = 0;i < files.length;i++)
         {
            if(files[i].isDirectory())
               dirs[idirs++] = files[i];
            else if(files[i].isFile())
               docs[idocs++] = files[i];
            else
               throw new NotImplementedException("Expecting file or directory");
         }
         if(!(target.exists() && target.isDirectory()))
            if(!target.mkdirs())
               messager.fatal(new XMException(Resources.getString("cannotcreatedirectory"),
                                              new Object[] {target.getAbsolutePath()}));
         for(int i = 0;i < idocs;i++)
         {
            String fname = docs[i].getName();
            int pos = fname.lastIndexOf('.');
            String extension = pos != -1 ? fname.substring(pos + 1) : "";
            if(extension.equals("xml"))
            {
               File result = style(docs[i],target);
               messager.progress(docs[i],result);
            }
            // else copy file
         }
         for(int i = 0;i < idirs;i++)
            walk(dirs[i],new File(target,dirs[i].getName()));
      }
      else
         messager.fatal(new XMException(Resources.getString("notdirectory"),
                                        new Object[] {source.getAbsolutePath()}));
   }

   /**
    * Style one file.
    * @param sourceFile the file to style
    * @param targetDir the directory to copy the result to
    * @return styled file or null if the file was ignored
    */
   protected File style(File sourceFile,File targetDir)
      throws XMException
   {
      try
      {
         String resultName = sourceFile.getName();
         int pos = resultName.lastIndexOf('.');
         if(pos != -1)
            resultName = resultName.substring(0,pos);
         resultName += extension;
         File resultFile = new File(targetDir,resultName);
         if(resultFile.exists())
         {
            if(resultFile.lastModified() >= sourceFile.lastModified() &&
               resultFile.lastModified() >= lastModified)
               return null;
         }
         Transformer transformer = templates.newTransformer();
         transformer.transform(new StreamSource(sourceFile),
                               new StreamResult(resultFile));
         return resultFile;
      }
      catch(TransformerException e)
      {
         throw new XMException(e);
      }
   }
}
